<?php

use GuzzleHttp\Client;
use GuzzleHttp\RequestOptions;
use GuzzleHttp\Exception\BadResponseException;

use Frisbo\FrisboSdk\Clients\V1\FrisboGuzzleClient;
use Frisbo\FrisboSdk\Exceptions\AccessForbiden;
use Frisbo\FrisboSdk\Exceptions\ResponseErrorHandler;
use Psr\Log\LoggerAwareTrait;

class FrisboClient extends FrisboGuzzleClient
{
    use LoggerAwareTrait, ResponseErrorHandler;

    const SUPPLY_STOCK = '/v1/stocks/{productId}/supply';
    const ORGANIZATION_WAREHOUSES_URL = '/v1/organizations/{organizationId}/warehouses';


    public function getOrganizationWarehouses(int $organizationId)
    {
        $accessToken = $this->getAccessToken();
        if (!$accessToken) {
            throw new AccessForbiden("Cannot access Frisbo Api.");
        }

        $client = new Client(['timeout'  => 30.0]);
        try {
            $response = $client->request(
                'GET',
                $this->apiUrl . str_replace('{organizationId}', $organizationId, self::ORGANIZATION_WAREHOUSES_URL),
                [
                    RequestOptions::HEADERS => ['Authorization' => 'Bearer ' . $accessToken]
                ]
            );

            return json_decode($response->getBody());
        } catch (BadResponseException $e) {
            $this->handleClientException($e, 'channel');
        } catch (\Exception $e) {
            $this->logger->info('FrisboClient::' . $e->getMessage());
            throw $e;
        }
    }

    public function supplyStock($productId, $body)
    {
        $accessToken = $this->getAccessToken();
        if (!$accessToken) {
            throw new AccessForbiden("Cannot access Frisbo Api.");
        }

        $client = new Client(['timeout'  => 30.0]);
        try {
            $response = $client->request(
                'POST',
                $this->apiUrl . str_replace(['{productId}'], [$productId], self::SUPPLY_STOCK),
                [
                    RequestOptions::HEADERS => ['Authorization' => 'Bearer ' . $accessToken],
                    RequestOptions::JSON => $body
                ]
            );

            return json_decode($response->getBody());
        } catch (BadResponseException $e) {
            $this->logger->info('FrisboClient::' . $e->getMessage());
            $this->handleClientException($e, 'storage');
        } catch (Exception $e) {
            $this->logger->info('FrisboClient::' . $e->getMessage());
            throw $e;
        }
    }
}
