<?php

namespace Frisbo\WooCommerce;

class FrisboPluginSettings
{
    /**
     * @var string
     */
    const OPTION_NAME = 'frisbo_data';

    public $email;
    public $password;
    public $organizationId;
    public $channelId;
    public $autoFulfillment;
    public $overwriteStock;
    public $noStock;
    public $useBillingForShipping;
    public $markOrderAsManual;
    public $productsAmount;
    public $stockAmount;
    public $ordersAmount;
    public $orderPrefix;
    public $organizations;
    public $channels;
    public $orderStatusSync;
    public $moveOrderToStatus;
    public $eanFields;
    public $vatRegNumberFields;
    public $lockerFields;
    public $currencyFields;
    public $packingNoteFields;
    public $shippingNameMapping;
    public $addressShippingFields;
    public $addressBillingFields;

    public function __construct(bool $reset = false)
    {
        $settings = \get_option(self::OPTION_NAME);
        $this->email = !empty($settings['email']) && !$reset ? $settings['email'] : null;
        $this->password = !empty($settings['password'])  && !$reset ? $settings['password'] : null;
        $this->organizationId = !empty($settings['organizationId'])  && !$reset ? $settings['organizationId'] : null;
        $this->channelId = !empty($settings['channelId']) && !$reset ? $settings['channelId'] : null;
        $this->organizations = !empty($settings['organizations'])  && !$reset ? $settings['organizations'] : null;
        $this->channels = !empty($settings['channels'])  && !$reset ? $settings['channels'] : null;
        $this->autoFulfillment = !empty($settings['auto-fulfillment']) ? $settings['auto-fulfillment'] : 'on';
        $this->overwriteStock = !empty($settings['overwrite-stock']) ? $settings['overwrite-stock'] : 'off';
        $this->noStock = !empty($settings['no-stock']) ? $settings['no-stock'] : 'off';
        $this->useBillingForShipping = !empty($settings['useBillingForShipping']) ? $settings['useBillingForShipping'] : null;
        $this->productsAmount = !empty($settings['products-amount']) ? $settings['products-amount'] : 50;
        $this->stockAmount = !empty($settings['stock-size']) ? $settings['stock-size'] : 10;
        $this->ordersAmount = !empty($settings['orders-amount']) ? $settings['orders-amount'] : 50;
        $this->orderPrefix = !empty($settings['order-prefix']) ? $settings['order-prefix'] : null;
        $this->orderStatusSync = !empty($settings['orderStatusSync']) ? $settings['orderStatusSync'] : null;
        $this->moveOrderToStatus = !empty($settings['moveOrderToStatus']) ? $settings['moveOrderToStatus'] : 'completed';
        $this->moveCanceledOrderToStatus = !empty($settings['moveCanceledOrderToStatus']) ? $settings['moveCanceledOrderToStatus'] : '-';
        $this->moveReturnedOrderToStatus = !empty($settings['moveReturnedOrderToStatus']) ? $settings['moveReturnedOrderToStatus'] : '-';
        $this->markOrderAsManual = !empty($settings['markOrderAsManual']) ? $settings['markOrderAsManual'] : 'bacs';
        $this->eanFields = !empty($settings['eanFields']) ? $settings['eanFields'] : null;
        $this->vatRegNumberFields = !empty($settings['vatRegNumberFields']) ? $settings['vatRegNumberFields'] : null;
        $this->lockerFields = !empty($settings['lockerFields']) ? $settings['lockerFields'] : null;
        $this->currencyFields = !empty($settings['currencyFields']) ? $settings['currencyFields'] : null;
        $this->packingNoteFields = !empty($settings['packingNoteFields']) ? $settings['packingNoteFields'] : null;
        $this->shippingNameMapping = !empty($settings['shippingNameMapping']) ? $settings['shippingNameMapping'] : null;
        $this->addressShippingFields = !empty($settings['addressShippingFields']) ? $settings['addressShippingFields'] : null;
        $this->addressBillingFields = !empty($settings['addressBillingFields']) ? $settings['addressBillingFields'] : null;
    }

    public function getSettings(): array
    {
        return [
            'email' => $this->email,
            'password' => $this->password,
            'organizationId' => $this->organizationId,
            'channelId' => $this->channelId,
            'autoFulfillment' => $this->autoFulfillment,
            'overwriteStock' => $this->overwriteStock,
            'noStock' => $this->noStock,
            'useBillingForShipping' => $this->useBillingForShipping,
            'productsAmount' => $this->productsAmount,
            'stockAmount' => $this->stockAmount,
            'ordersAmount' => $this->ordersAmount,
            'orderPrefix' => $this->orderPrefix,
            'organizations' => $this->organizations,
            'channels' => $this->channels,
            'orderStatusSync' => $this->orderStatusSync,
            'moveOrderToStatus' => $this->moveOrderToStatus,
            'moveCanceledOrderToStatus' => $this->moveCanceledOrderToStatus,
            'moveReturnedOrderToStatus' => $this->moveReturnedOrderToStatus,
            'markOrderAsManual' => $this->markOrderAsManual,
            'eanFields' => $this->eanFields,
            'vatRegNumberFields' => $this->vatRegNumberFields,
            'lockerFields' => $this->lockerFields,
            'currencyFields' => $this->currencyFields,
            'packingNoteFields' => $this->packingNoteFields,
            'shippingNameMapping' => $this->shippingNameMapping,
            'addressShippingFields' => $this->addressShippingFields,
            'addressBillingFields' => $this->addressBillingFields,

            //TODO: to be removed
            'auto-fulfillment' => $this->autoFulfillment,
            'overwrite-stock' => $this->overwriteStock,
            'no-stock' => $this->noStock,
            'products-amount' => $this->productsAmount,
            'stock-size' => $this->stockAmount,
            'orders-amount' => $this->ordersAmount,
            'order-prefix' => $this->orderPrefix,
        ];
    }

    public function updateSettings()
    {
        \update_option(self::OPTION_NAME, $this->getSettings());
        Frisbo::instance()->refreshSettings();
    }
}
