<?php
namespace Frisbo\WooCommerce;

use Frisbo\FrisboSdk\Models\PackageProduct;
use Frisbo\FrisboSdk\Models\Product;
use Frisbo\FrisboSdk\Models\ProductDimensions;

class FrisboProductDto
{
    use FrisboTrait;

    private $product;
    private $frisbo;


    public function __construct(\WC_Product $product)
    {
        $this->product = $product;
        $this->frisbo = Frisbo::instance();
    }

    public function getMappings()
    {
        $productType = $this->product->get_type();

        if ($productType == 'variation') {
            $parentProduct = \wc_get_product($this->product->get_parent_id());
            if ($this->product->get_sku() == $parentProduct->get_sku()) {
                $this->product->set_sku(str_replace('-', '_', $this->product->get_slug()));
            }
        }

        $productMappings = new Product();
        $productMappings->type = 'product';

        $productMappings->has_serial_number = 0;
        $productMappings->name = $this->product->get_name();
        $productMappings->sku = $this->product->get_sku();
        $productMappings->external_code = $this->product->get_sku();
        $productMappings->upc = $this->product->get_sku();
        $productMappings->vat = 0;
        $productMappings->ean = $this->getEan($this->product);

        $productMappings->dimensions = $this->getProductDimensions($this->product);

        return $productMappings;
    }

    private function getProductDimensions(\WC_Product $product): ProductDimensions
    {
        $dimensions = new ProductDimensions();
        $dimensions->height = !empty($product->get_height()) ? $product->get_height() : 0.1;
        $dimensions->width = !empty($product->get_width()) ? $product->get_width() : 0.1;
        $dimensions->length = !empty($product->get_length()) ? $product->get_length() : 0.1;
        $dimensions->weight = !empty($product->get_weight()) ? $product->get_weight() : 0.1;
        return $dimensions;
    }

    private function getEan(\WC_Product $product): string
    {
        $eanSettings = $this->frisbo->settings->eanFields ? explode(',', $this->frisbo->settings->eanFields) : [];
        $eanFields = array_merge(['_woosea_ean'], $eanSettings);
        foreach ($eanFields as $field) {
            $ean = \get_post_meta($product->get_id(), trim($field), true);
            if (!empty($ean)) {
                return $ean;
            }
        }
        return $product->get_sku();
    }
}
