<?php

namespace Frisbo\WooCommerce;

trait WoocommerceTrait
{
    use FrisboTrait;

    private function createProduct($options = null): \WC_Product
    {
        $identifier = $this->getRand();
        $product = new \WC_Product_Simple();
        $product->set_name('Test Product ' . $identifier);
        $product->set_status('publish');
        $product->set_catalog_visibility('visible');
        $product->set_price($this->getRandFloat());
        $product->set_sku(!empty($options['sku']) ? (empty($options['unique']) ? $options['sku'] . $identifier : $options['sku']) : $identifier);
        if (!empty($options['isVirtual'])) {
            $product->set_virtual(true);
        }
        $product->save();
        return $product;
    }

    private function getProduct($identifier, $isSku = false)
    {
        if ($isSku) {
            return wc_get_product_id_by_sku($identifier);
        }
        return wc_get_product($identifier);
    }

    private function createOrder($options)
    {
        $address = [
            'first_name' => 'Filip',
            'last_name'  => 'Test',
            'email'      => 'test@frisbo.ro',
            'phone'      => '0723456789',
            'address_1'  => 'str. Test nr. 1',
            'address_2'  => '',
            'city'       => 'Cluj-Napoca',
            'state'      => 'Cluj',
            'postcode'   => '400000',
            'country'    => 'RO'
        ];

        if (!empty($options['company'])) {
            $address['company'] = $options['company'];
        }

        $order = wc_create_order();

        $order->set_address($address, 'billing');
        $order->set_address($address, 'shipping');

        $paymentGateways = WC()->payment_gateways->payment_gateways();
        $order->set_payment_method($paymentGateways['cod']);
        $order->remove_order_items();

        foreach ($options['products'] as $product) {
            $order->add_product($product);
        }

        $order->calculate_totals();
        $order->update_status('processing');

        return $order;
    }

    private function getOrder($identifier)
    {
        return wc_get_order($identifier);
    }
}
