<?php

use Frisbo\FrisboSdk\Models\Filters\Filter;

require_once FRISBO_PATH . '/app/FrisboEnvs.php';
require_once FRISBO_TESTS . '/WoocommerceTrait.php';

class AjaxFrisboWoocommerce extends WP_Ajax_UnitTestCase
{

    use WoocommerceTrait;

    public function testCredentials()
    {
        $frisbo = Frisbo::instance();
        $_POST['email'] = FrisboEnvs::EMAIL_2;
        $_POST['password'] = FrisboEnvs::PASSWORD_2;

        /**
         * Exception: Cannot modify header information - headers already sent
         * HACK: Comment out wp-includes/functions.php:6821 & wp-admin/includes/misc.php:1310
         */
        try {
            $this->_handleAjax('frisbo_login');
        } catch (Exception $ex) {
            $this->log($this->exceptionInfo($ex));
        }

        $this->assertEquals($_POST['email'], $frisbo->settings->email);
        $this->assertEquals($_POST['password'], $frisbo->settings->password);

        $frisbo->setClient($this->settings->email, $this->settings->password);
    }

    public function testDefaultSettings()
    {
        $frisbo = Frisbo::instance();
        $organizations = $frisbo->client->getOrganizations();
        $firstOrganizationId = $organizations[0]->organization_id;
        $channels = $frisbo->client->getOrganizationChannels($firstOrganizationId);
        $firstChannelId = $channels[0]->id;

        $this->assertEquals($organizations, $frisbo->settings->organizations);
        $this->assertEquals($channels, $frisbo->settings->channels);
        $this->assertEquals($firstOrganizationId, $frisbo->settings->organizationId);
        $this->assertEquals($firstChannelId, $frisbo->settings->channelId);

        $this->assertEquals('on', $frisbo->settings->autoFulfillment);
        $this->assertEquals('off', $frisbo->settings->overwriteStock);
        $this->assertEquals('off', $frisbo->settings->noStock);
        $this->assertEquals('off', $frisbo->settings->virtualBundle);
        $this->assertEquals(50, $frisbo->settings->productsAmount);
        $this->assertEquals(10, $frisbo->settings->stockAmount);
        $this->assertEquals(50, $frisbo->settings->ordersAmount);
        $this->assertEquals(null, $frisbo->settings->orderPrefix);
        $this->assertEquals(null, $frisbo->settings->orderStatusSync);
        $this->assertEquals(null, $frisbo->settings->eanFields);
        $this->assertEquals(null, $frisbo->settings->vatRegNumberFields);
        $this->assertEquals(null, $frisbo->settings->lockerFields);
        $this->assertEquals(null, $frisbo->settings->currencyFields);
    }

    public function testUpdateSettings()
    {
        $frisbo = Frisbo::instance();
        $_POST['overwriteStock'] = 'on';
        $_POST['orderPrefix'] = '_FRIS';
        $_POST['productsAmount'] = 100;
        $_POST['currencyFields'] = 'currency_field_1';

        $this->_handleAjax('frisbo_update_plugin_settings');

        $this->assertEquals($_POST['overwriteStock'], $frisbo->settings->overwriteStock);
        $this->assertEquals($_POST['orderPrefix'], $frisbo->settings->orderPrefix);
        $this->assertEquals($_POST['productsAmount'], $frisbo->settings->productsAmount);
        $this->assertEquals($_POST['currencyFields'], $frisbo->settings->currencyFields);
    }

    /**
     * When changing the user we set the settings to default
     */
    public function testChangeUser()
    {
        $frisbo = Frisbo::instance();
        $_POST['email'] = FrisboEnvs::EMAIL_2;
        $_POST['password'] = FrisboEnvs::PASSWORD_2;

        $this->_handleAjax('frisbo_login');

        $_POST['autoFulfillment'] = 'on';
        $_POST['overwriteStock'] = 'on';
        $_POST['orderPrefix'] = $this->getRand();

        $this->_handleAjax('frisbo_update_plugin_settings');

        $organizations = $frisbo->client->getOrganizations();
        $firstOrganizationId = $organizations[0]->organization_id;
        $channels = $frisbo->client->getOrganizationChannels($firstOrganizationId);
        $firstChannelId = $channels[0]->id;

        // Force reset these settings on user change
        $this->assertEquals($organizations, $frisbo->settings->organizations);
        $this->assertEquals($channels, $frisbo->settings->channels);
        $this->assertEquals($firstOrganizationId, $frisbo->settings->organizationId);
        $this->assertEquals($firstChannelId, $frisbo->settings->channelId);

        $this->assertEquals($_POST['email'], $frisbo->settings->email);
        $this->assertEquals($_POST['password'], $frisbo->settings->password);
        $this->assertEquals($_POST['autoFulfillment'], $frisbo->settings->autoFulfillment);
        $this->assertEquals($_POST['overwriteStock'], $frisbo->settings->overwriteStock);
        $this->assertEquals($_POST['orderPrefix'], $frisbo->settings->orderPrefix);
    }

    public function syncProductsButton()
    {
        $frisbo = Frisbo::instance();
        $products[] = $this->createProduct();
        $products[] = $this->createProduct();
        $products[] = $this->createProduct();

        $this->_handleAjax('frisbo_sync_products');

        foreach ($products as $product) {
            $frisboProduct = $this->waitFrisboProduct($product->get_sku());
            $product = $this->getProduct($product->get_id());
            $this->assertEquals($frisboProduct->product_id, $product->get_meta('frisbo_product_id'));
        }
        return $products;
    }

    /**
     * This test will pass only if user has Administrator role.
     * 
     */
    public function syncStockButton()
    {
        $frisbo = Frisbo::instance();
        $frisbo->settings->overwriteStock = 'on';
        $frisbo->settings->updateSettings();

        $products = $this->syncProductsButton();

        $quantities = [];
        foreach ($products as $product) {
            $frisboProductId = $product->get_meta('frisbo_product_id');
            $warehouses = $frisbo->client->getOrganizationWarehouses($frisbo->settings->organizationId);

            $quantity = $this->getRand(true);
            $frisbo->client->supplyStock($frisboProductId, [
                'warehouse_id' => $warehouses[0]->id,
                'quantity' => $quantity
            ]);
            $quantities[$product->get_id()] = $quantity;
        }

        $this->_handleAjax('frisbo_sync_stock');

        foreach ($products as $product) {
            $product = $this->getProduct($product->get_id());
            $this->assertEquals($quantities[$product->get_id()], $product->get_stock_quantity());
        }
        return $products;
    }

    /**
     */
    public function syncOrdersButton()
    {
        $frisbo = Frisbo::instance();
        $prefix = '_' . $this->getRand();
        $frisbo->settings->orderPrefix = $prefix;
        $frisbo->settings->updateSettings();
        $products = $this->syncStockButton();

        $orders[] = $this->createOrder([
            'products' => $products
        ]);
        $orders[] = $this->createOrder([
            'products' => $products
        ]);
        $orders[] = $this->createOrder([
            'products' => $products
        ]);

        $this->_handleAjax('frisbo_sync_orders');

        foreach ($orders as $order) {
            $frisboOrder = $this->waitFrisboOrder($order->get_order_number());
            $order = $this->getOrder($order->get_id());
            $this->assertEquals($frisboOrder->order_id, $order->get_meta('frisbo_order_number'));
            $orderTotal = 0;
            foreach ($order->get_items() as $itemValues) {
                $orderTotal += $this->getPrice($itemValues);
            }
            $frisboOrderTotal = 0;
            foreach ($frisboOrder->products as $itemValues) {
                $frisboOrderTotal += $itemValues->total;
            }
            $this->assertEquals($orderTotal, $frisboOrderTotal);
        }
        return $orders;
    }

    private function getPrice(WC_Order_Item_Product $itemValues): float
    {
        return ($itemValues->get_subtotal() + $itemValues->get_subtotal_tax()) * $itemValues->get_quantity();
    }

    /**
     */
    public function testSyncStatusButton()
    {
        $frisbo = Frisbo::instance();
        $orders = $this->syncOrdersButton();
        $shippedOrders = [];
        foreach ($orders as $order) {
            $frisboOrder = $this->waitFrisboOrder($order->get_order_number(), ['invoiced']);
            $frisbo->client->transitionOrder($frisbo->settings->organizationId, $frisboOrder, 'shipOrder');
            $shippedOrders[$order->get_id()] = $this->waitFrisboOrder($order->get_order_number(), ['shipped']);
        }

        $this->_handleAjax('frisbo_sync_status');

        foreach ($orders as $order) {
            $order = $this->getOrder($order->get_id());
            $this->assertEquals($shippedOrders[$order->get_id()]->shipping_tracking_number, $order->get_meta('frisbo_awb_number'));
            $this->assertEquals($shippedOrders[$order->get_id()]->tracking_url, $order->get_meta('frisbo_awb_tracking_url'));
        }
    }
}
